/*************************************************************************/
/*************************************************************************/
/* H. Timucin OZDEMIR                                                    */
/* Date : April, 1997                                                    */
/* File Name : JSSProblem.cc                                             */
/*************************************************************************/
/*************************************************************************/

#include <stdio.h>
#include <iostream.h>
#include "JSSProblem.h"
#include "OperationList.h"

JSSProblem::~JSSProblem()
{  
   NumberOfJobs       = -1;
   NumberOfFacilities = -1;
   cout << " de allocate JSSProblem " << endl;
   if((JobIndex>0)&&(JobList!=NULL)) delete [] JobList;
   if((FacilityIndex>0)&&(FacilityScheduleList!=NULL)) 
	      delete [] FacilityScheduleList;
   JobIndex      = 0;
   FacilityIndex = 0;
   cout << " end of de allocate JSSProblem " << endl;
}// end of constructor()

JSSProblem::JSSProblem()
{  NumberOfJobs       = -1;
   NumberOfFacilities = -1;
   JobIndex      = 0;
   FacilityIndex = 0;
   JobList = NULL;
   FacilityScheduleList = NULL;
}// end of constructor()

JSSProblem::JSSProblem(int _NJobs, int _NFac)
{  NumberOfJobs       = _NJobs;
   NumberOfFacilities = _NFac;
   JobIndex      = 0;
   FacilityIndex = 0;
   JobList = NULL;
   FacilityScheduleList = NULL;
}// end of constructor()

JSSProblem::JSSProblem(const JSSProblem& RHS)
{  
   NumberOfJobs       = RHS.NumberOfJobs;
   NumberOfFacilities = RHS.NumberOfFacilities;
   JobIndex      = RHS.JobIndex;
   FacilityIndex = RHS.FacilityIndex;

   JobList = new OperationList[JobIndex];
   for(int i=0;i<JobIndex;i++) 
       JobList[i] = RHS.JobList[i];

   FacilityScheduleList = new FacilitySchedule[FacilityIndex];
   for(i=0;i<FacilityIndex;i++) 
       FacilityScheduleList[i] = RHS.FacilityScheduleList[i];

}// end of COPY constructor()

JSSProblem::addJob(OperationList Job)
{   
    OperationList *Temp = new OperationList[JobIndex+1];
    int i;

    for(i=0;i<JobIndex;i++) Temp[i]=JobList[i];

    if((JobIndex>0)&&(JobList!=NULL)) delete [] JobList;

    JobList = new OperationList[JobIndex+1];

    for(i=0;i<JobIndex;i++) JobList[i] = Temp[i];

    JobIndex++;

    JobList[i] = Job;

    delete [] Temp;

    return JobIndex;

}// end of addJob()

void JSSProblem::setNumberOfJobs(int _NumJobs)
{  NumberOfJobs = _NumJobs; 
}// end of setNumberOfJobs()

void JSSProblem::setNumberOfFacilities(int _NumFac)
{  NumberOfFacilities = _NumFac;
}// end of setNumberOfFacilities()

JSSProblem::getNumberOfJobs()
{  return NumberOfJobs ;
}// end of getNumberOfJobs()

JSSProblem::getNumberOfFacilities()
{  return NumberOfFacilities;
}// end of getNumberOfFacilities()

OperationList JSSProblem::getJobList(int indx)
{   OperationList *ret = new OperationList[1];

    return (indx>=0 && indx<JobIndex?JobList[indx]:(*ret));

}// end of getJobList()

/*
//  Returns the address of the previus operation OP on job indx.
*/

Operation * JSSProblem::getPreviousOperation(int indx, Operation OP)
{   Operation * ret = NULL;
    
    if(indx>=00 && indx<JobIndex)
    {  int i, numOps;
       int notFound=1;
       numOps = JobList[indx].getNumberOfOperation();

       //* find the operation that we are looking for

       for(i=0; notFound && i<numOps;i++)
         if(*(JobList[indx].getOperationWithIndex(i))==OP) notFound=0;
	 
       //* If it is not the first operation 

       if( !notFound && i>0 )  ret = JobList[indx].getOperationWithIndex(i-1); 

    }// end of if

    return ret;

}// end of getPreviousOperation()

/*
//  Returns the address of the following operation OP on job indx.
*/

Operation * JSSProblem::getFollowingOperation(int indx, Operation OP)
{   Operation * ret = NULL;
    
    if(indx>=0 && indx<JobIndex)
    {  int i, numOps;
       int notFound=1;
       numOps = JobList[indx].getNumberOfOperation();

       //* find the operation that we are looking for

       for(i=0; notFound && i<numOps;i++)
         if(*(JobList[indx].getOperationWithIndex(i))==OP) notFound=0;
	 
       //* If it is not the first operation 

       if( !notFound && i<(numOps-1) ) 
	   ret = JobList[indx].getOperationWithIndex(i+1); 

    }// end of if

    return ret;

}// end of getFollowingOperation()

void JSSProblem::print()
{  //operator << (cout , (*this));
   cout << (*this);
}// end of print()

JSSProblem JSSProblem::operator =(const JSSProblem &RHSOperation)
{  int i;

   NumberOfJobs       = RHSOperation.NumberOfJobs       ;
   NumberOfFacilities = RHSOperation.NumberOfFacilities ;
   
   if(JobIndex) delete [] JobList;
   if(FacilityIndex) delete [] FacilityScheduleList;

   JobIndex      = RHSOperation.JobIndex      ;
   FacilityIndex = RHSOperation.FacilityIndex ;

   JobList = new OperationList[JobIndex];
   FacilityScheduleList = new FacilitySchedule[FacilityIndex];

   for(i=0;i<JobIndex;i++) JobList[i] = RHSOperation.JobList[i];
   for(i=0;i<FacilityIndex;i++) 
	FacilityScheduleList[i] = RHSOperation.FacilityScheduleList[i];

   return (*this);

}// end of operator =()

ostream & operator <<(ostream &os, const JSSProblem &RHSPair)
{
   os << "Number of Facilities " << RHSPair.NumberOfFacilities << endl;
   os << "Number of Jobs       " << RHSPair.NumberOfJobs << endl;

   for(int i=0;i<RHSPair.JobIndex;i++)
      os << endl << "Job " << i << " ==> " << RHSPair.JobList[i];

   return os;

}// end of operator <<

